@@ -17,20 +17,16 @@ each function for more information.
 from __future__ import print_function, division
 
 from sympy.core import Dummy, ilcm, Add, Mul, Pow, S
-
-from sympy.matrices import Matrix, zeros, eye
-
-from sympy.solvers import solve
-
-from sympy.polys import Poly, lcm, cancel, sqf_list
-
-from sympy.integrals.risch import (gcdex_diophantine, frac_in, derivation,
-    NonElementaryIntegralException, residue_reduce, splitfactor,
-    residue_reduce_derivation, DecrementLevel, recognize_log_derivative)
-from sympy.integrals.rde import (order_at, order_at_oo, weak_normalizer,
-    bound_degree, spde, solve_poly_rde)
 from sympy.core.compatibility import reduce, range
-from sympy.utilities.misc import debug
+from sympy.integrals.rde import (order_at, order_at_oo, weak_normalizer,
+    bound_degree)
+from sympy.integrals.risch import (gcdex_diophantine, frac_in, derivation,
+    residue_reduce, splitfactor, residue_reduce_derivation, DecrementLevel,
+    recognize_log_derivative)
+from sympy.matrices import zeros, eye
+from sympy.polys import Poly, lcm, cancel, sqf_list
+from sympy.polys.polymatrix import PolyMatrix as Matrix
+from sympy.solvers import solve
 
 
 def prde_normal_denom(fa, fd, G, DE):
@@ -90,7 +86,7 @@ def prde_special_denom(a, ba, bd, G, DE, case='auto'):
     """
     Parametric Risch Differential Equation - Special part of the denominator.
 
-    case is on of {'exp', 'tan', 'primitive'} for the hyperexponential,
+    case is one of {'exp', 'tan', 'primitive'} for the hyperexponential,
     hypertangent, and primitive cases, respectively.  For the hyperexponential
     (resp. hypertangent) case, given a derivation D on k[t] and a in k[t],
     b in k<t>, and g1, ..., gm in k(t) with Dt/t in k (resp. Dt/(t**2 + 1) in
@@ -131,8 +127,8 @@ def prde_special_denom(a, ba, bd, G, DE, case='auto'):
                 etaa, etad = frac_in(dcoeff, DE.t)
                 A = parametric_log_deriv(alphaa, alphad, etaa, etad, DE)
                 if A is not None:
-                    a, m, z = A
-                    if a == 1:
+                    Q, m, z = A
+                    if Q == 1:
                         n = min(n, m)
 
         elif case == 'tan':
@@ -146,9 +142,10 @@ def prde_special_denom(a, ba, bd, G, DE, case='auto'):
                     A = parametric_log_deriv(alphaa, alphad, etaa, etad, DE)
                     B = parametric_log_deriv(betaa, betad, etaa, etad, DE)
                     if A is not None and B is not None:
-                        a, s, z = A
-                        if a == 1:
-                             n = min(n, s/2)
+                        Q, s, z = A
+                        # TODO: Add test
+                        if Q == 1:
+                            n = min(n, s/2)
 
     N = max(0, -nb)
     pN = p**N
@@ -188,11 +185,29 @@ def prde_linear_constraints(a, b, G, DE):
         N = max([ri.degree(DE.t) for _, ri in Q])
         M = Matrix(N + 1, m, lambda i, j: Q[j][1].nth(i))
     else:
-        M = Matrix()  # No constraints, return the empty matrix.
+        M = Matrix(0, m, [])  # No constraints, return the empty matrix.
 
     qs, _ = list(zip(*Q))
     return (qs, M)
 
+def poly_linear_constraints(p, d):
+    """
+    Given p = [p1, ..., pm] in k[t]^m and d in k[t], return
+    q = [q1, ..., qm] in k[t]^m and a matrix M with entries in k such
+    that Sum(ci*pi, (i, 1, m)), for c1, ..., cm in k, is divisible
+    by d if and only if (c1, ..., cm) is a solution of Mx = 0, in
+    which case the quotient is Sum(ci*qi, (i, 1, m)).
+    """
+    m = len(p)
+    q, r = zip(*[pi.div(d) for pi in p])
+
+    if not all([ri.is_zero for ri in r]):
+        n = max([ri.degree() for ri in r])
+        M = Matrix(n + 1, m, lambda i, j: r[j].nth(i))
+    else:
+        M = Matrix(0, m, [])  # No constraints.
+
+    return q, M
 
 def constant_system(A, u, DE):
     """
@@ -216,7 +231,7 @@ def constant_system(A, u, DE):
     if not A:
         return A, u
     Au = A.row_join(u)
-    Au = Au.rref(simplify=cancel)[0]
+    Au = Au.rref(simplify=cancel, normalize_last=False)[0]
     # Warning: This will NOT return correct results if cancel() cannot reduce
     # an identically zero expression to 0.  The danger is that we might
     # incorrectly prove that an integral is nonelementary (such as
@@ -294,9 +309,9 @@ def prde_no_cancel_b_large(b, Q, n, DE):
 
     Given a derivation D on k[t], n in ZZ, and b, q1, ..., qm in k[t] with
     b != 0 and either D == d/dt or deg(b) > max(0, deg(D) - 1), returns
-    h1, ..., hr in k[r] and a matrix A with coefficients in Const(k) such that
+    h1, ..., hr in k[t] and a matrix A with coefficients in Const(k) such that
     if c1, ..., cm in Const(k) and q in k[t] satisfy deg(q) <= n and
-    Dq + b*Q == Sum(ci*qi, (i, 1, m)), then q = Sum(dj*hj, (j, 1, r)), where
+    Dq + b*q == Sum(ci*qi, (i, 1, m)), then q = Sum(dj*hj, (j, 1, r)), where
     d1, ..., dr in Const(k) and A*Matrix([[c1, ..., cm, d1, ..., dr]]).T == 0.
     """
     db = b.degree(DE.t)
@@ -314,7 +329,7 @@ def prde_no_cancel_b_large(b, Q, n, DE):
         dc = -1
         M = zeros(0, 2)
     else:
-        dc = max([qi.degree(t) for qi in Q])
+        dc = max([qi.degree(DE.t) for qi in Q])
         M = Matrix(dc + 1, m, lambda i, j: Q[j].nth(i))
     A, u = constant_system(M, zeros(dc + 1, 1), DE)
     c = eye(m)
@@ -359,35 +374,374 @@ def prde_no_cancel_b_small(b, Q, n, DE):
         c = eye(m)
         A = A.row_join(zeros(A.rows, m)).col_join(c.row_join(-c))
         return (H, A)
+
+    # else: b is in k, deg(qi) < deg(Dt)
+
+    t = DE.t
+    if DE.case != 'base':
+        with DecrementLevel(DE):
+            t0 = DE.t  # k = k0(t0)
+            ba, bd = frac_in(b, t0, field=True)
+            Q0 = [frac_in(qi.TC(), t0, field=True) for qi in Q]
+            f, B = param_rischDE(ba, bd, Q0, DE)
+
+            # f = [f1, ..., fr] in k^r and B is a matrix with
+            # m + r columns and entries in Const(k) = Const(k0)
+            # such that Dy0 + b*y0 = Sum(ci*qi, (i, 1, m)) has
+            # a solution y0 in k with c1, ..., cm in Const(k)
+            # if and only y0 = Sum(dj*fj, (j, 1, r)) where
+            # d1, ..., dr ar in Const(k) and
+            # B*Matrix([c1, ..., cm, d1, ..., dr]) == 0.
+
+        # Transform fractions (fa, fd) in f into constant
+        # polynomials fa/fd in k[t].
+        # (Is there a better way?)
+        f = [Poly(fa.as_expr()/fd.as_expr(), t, field=True)
+             for fa, fd in f]
     else:
-        # TODO: implement this (requires recursive param_rischDE() call)
-        raise NotImplementedError
+        # Base case. Dy == 0 for all y in k and b == 0.
+        # Dy + b*y = Sum(ci*qi) is solvable if and only if
+        # Sum(ci*qi) == 0 in which case the solutions are
+        # y = d1*f1 for f1 = 1 and any d1 in Const(k) = k.
+
+        f = [Poly(1, t, field=True)]  # r = 1
+        B = Matrix([[qi.TC() for qi in Q] + [S.Zero]])
+        # The condition for solvability is
+        # B*Matrix([c1, ..., cm, d1]) == 0
+        # There are no constraints on d1.
+
+    # Coefficients of t^j (j > 0) in Sum(ci*qi) must be zero.
+    d = max([qi.degree(DE.t) for qi in Q])
+    if d > 0:
+        M = Matrix(d, m, lambda i, j: Q[j].nth(i + 1))
+        A, _ = constant_system(M, zeros(d, 1), DE)
+    else:
+        # No constraints on the hj.
+        A = Matrix(0, m, [])
+
+    # Solutions of the original equation are
+    #    y = Sum(dj*fj, (j, 1, r) + Sum(ei*hi, (i, 1, m)),
+    # where  ei == ci  (i = 1, ..., m),  when
+    # A*Matrix([c1, ..., cm]) == 0 and
+    # B*Matrix([c1, ..., cm, d1, ..., dr]) == 0
+
+    # Build combined constraint matrix with m + r + m columns.
+
+    r = len(f)
+    I = eye(m)
+    A = A.row_join(zeros(A.rows, r + m))
+    B = B.row_join(zeros(B.rows, m))
+    C = I.row_join(zeros(m, r)).row_join(-I)
+
+    return f + H, A.col_join(B).col_join(C)
+
+
+def prde_cancel_liouvillian(b, Q, n, DE):
+    """
+    Pg, 237.
+    """
+    H = []
+
+    # Why use DecrementLevel? Below line answers that:
+    # Assuming that we can solve such problems over 'k' (not k[t])
+    if DE.case == 'primitive':
+        with DecrementLevel(DE):
+            ba, bd = frac_in(b, DE.t, field=True)
+
+    for i in range(n, -1, -1):
+        if DE.case == 'exp': # this re-checking can be avoided
+            with DecrementLevel(DE):
+                ba, bd = frac_in(b + i*derivation(DE.t, DE)/DE.t,
+                                DE.t, field=True)
+        with DecrementLevel(DE):
+            Qy = [frac_in(q.nth(i), DE.t, field=True) for q in Q]
+            fi, Ai = param_rischDE(ba, bd, Qy, DE)
+        fi = [Poly(fa.as_expr()/fd.as_expr(), DE.t, field=True)
+                for fa, fd in fi]
+
+        ri = len(fi)
+
+        if i == n:
+            M = Ai
+        else:
+            M = Ai.col_join(M.row_join(zeros(M.rows, ri)))
+
+        Fi, hi = [None]*ri, [None]*ri
+
+        # from eq. on top of p.238 (unnumbered)
+        for j in range(ri):
+            hji = fi[j]*DE.t**i
+            hi[j] = hji
+            # building up Sum(djn*(D(fjn*t^n) - b*fjnt^n))
+            Fi[j] = -(derivation(hji, DE) - b*hji)
+
+        H += hi
+        # in the next loop instead of Q it has
+        # to be Q + Fi taking its place
+        Q = Q + Fi
+
+    return (H, M)
+
+
+def param_poly_rischDE(a, b, q, n, DE):
+    """Polynomial solutions of a parametric Risch differential equation.
+
+    Given a derivation D in k[t], a, b in k[t] relatively prime, and q
+    = [q1, ..., qm] in k[t]^m, return h = [h1, ..., hr] in k[t]^r and
+    a matrix A with m + r columns and entries in Const(k) such that
+    a*Dp + b*p = Sum(ci*qi, (i, 1, m)) has a solution p of degree <= n
+    in k[t] with c1, ..., cm in Const(k) if and only if p = Sum(dj*hj,
+    (j, 1, r)) where d1, ..., dr are in Const(k) and (c1, ..., cm,
+    d1, ..., dr) is a solution of Ax == 0.
+    """
+    m = len(q)
+    if n < 0:
+        # Only the trivial zero solution is possible.
+        # Find relations between the qi.
+        if all([qi.is_zero for qi in q]):
+            return [], zeros(1, m)  # No constraints.
+
+        N = max([qi.degree(DE.t) for qi in q])
+        M = Matrix(N + 1, m, lambda i, j: q[j].nth(i))
+        A, _ = constant_system(M, zeros(M.rows, 1), DE)
+
+        return [], A
+
+    if a.is_ground:
+        # Normalization: a = 1.
+        a = a.LC()
+        b, q = b.quo_ground(a), [qi.quo_ground(a) for qi in q]
+
+        if not b.is_zero and (DE.case == 'base' or
+                b.degree() > max(0, DE.d.degree() - 1)):
+            return prde_no_cancel_b_large(b, q, n, DE)
+
+        elif ((b.is_zero or b.degree() < DE.d.degree() - 1)
+                and (DE.case == 'base' or DE.d.degree() >= 2)):
+            return prde_no_cancel_b_small(b, q, n, DE)
+
+        elif (DE.d.degree() >= 2 and
+              b.degree() == DE.d.degree() - 1 and
+              n > -b.as_poly().LC()/DE.d.as_poly().LC()):
+            raise NotImplementedError("prde_no_cancel_b_equal() is "
+                "not yet implemented.")
+
+        else:
+            # Liouvillian cases
+            if DE.case == 'primitive' or DE.case == 'exp':
+                return prde_cancel_liouvillian(b, q, n, DE)
+            else:
+                raise NotImplementedError("non-linear and hypertangent "
+                        "cases have not yet been implemented")
+
+    # else: deg(a) > 0
+
+    # Iterate SPDE as long as possible cumulating coefficient
+    # and terms for the recovery of original solutions.
+    alpha, beta = 1, [0]*m
+    while n >= 0:  # and a, b relatively prime
+        a, b, q, r, n = prde_spde(a, b, q, n, DE)
+        beta = [betai + alpha*ri for betai, ri in zip(beta, r)]
+        alpha *= a
+        # Solutions p of a*Dp + b*p = Sum(ci*qi) correspond to
+        # solutions alpha*p + Sum(ci*betai) of the initial equation.
+        d = a.gcd(b)
+        if not d.is_ground:
+            break
+
+    # a*Dp + b*p = Sum(ci*qi) may have a polynomial solution
+    # only if the sum is divisible by d.
+
+    qq, M = poly_linear_constraints(q, d)
+    # qq = [qq1, ..., qqm] where qqi = qi.quo(d).
+    # M is a matrix with m columns an entries in k.
+    # Sum(fi*qi, (i, 1, m)), where f1, ..., fm are elements of k, is
+    # divisible by d if and only if M*Matrix([f1, ..., fm]) == 0,
+    # in which case the quotient is Sum(fi*qqi).
+
+    A, _ = constant_system(M, zeros(M.rows, 1), DE)
+    # A is a matrix with m columns and entries in Const(k).
+    # Sum(ci*qqi) is Sum(ci*qi).quo(d), and the remainder is zero
+    # for c1, ..., cm in Const(k) if and only if
+    # A*Matrix([c1, ...,cm]) == 0.
+
+    V = A.nullspace()
+    # V = [v1, ..., vu] where each vj is a column matrix with
+    # entries aj1, ..., ajm in Const(k).
+    # Sum(aji*qi) is divisible by d with exact quotient Sum(aji*qqi).
+    # Sum(ci*qi) is divisible by d if and only if ci = Sum(dj*aji)
+    # (i = 1, ..., m) for some d1, ..., du in Const(k).
+    # In that case, solutions of
+    #     a*Dp + b*p = Sum(ci*qi) = Sum(dj*Sum(aji*qi))
+    # are the same as those of
+    #     (a/d)*Dp + (b/d)*p = Sum(dj*rj)
+    # where rj = Sum(aji*qqi).
+
+    if not V:  # No non-trivial solution.
+        return [], eye(m)  # Could return A, but this has
+                           # the minimum number of rows.
+
+    Mqq = Matrix([qq])  # A single row.
+    r = [(Mqq*vj)[0] for vj in V]  # [r1, ..., ru]
+
+    # Solutions of (a/d)*Dp + (b/d)*p = Sum(dj*rj) correspond to
+    # solutions alpha*p + Sum(Sum(dj*aji)*betai) of the initial
+    # equation. These are equal to alpha*p + Sum(dj*fj) where
+    # fj = Sum(aji*betai).
+    Mbeta = Matrix([beta])
+    f = [(Mbeta*vj)[0] for vj in V]  # [f1, ..., fu]
+
+    #
+    # Solve the reduced equation recursively.
+    #
+    g, B = param_poly_rischDE(a.quo(d), b.quo(d), r, n, DE)
+
+    # g = [g1, ..., gv] in k[t]^v and and B is a matrix with u + v
+    # columns and entries in Const(k) such that
+    # (a/d)*Dp + (b/d)*p = Sum(dj*rj) has a solution p of degree <= n
+    # in k[t] if and only if p = Sum(ek*gk) where e1, ..., ev are in
+    # Const(k) and B*Matrix([d1, ..., du, e1, ..., ev]) == 0.
+    # The solutions of the original equation are then
+    # Sum(dj*fj, (j, 1, u)) + alpha*Sum(ek*gk, (k, 1, v)).
+
+    # Collect solution components.
+    h = f + [alpha*gk for gk in g]
+
+    # Build combined relation matrix.
+    A = -eye(m)
+    for vj in V:
+        A = A.row_join(vj)
+    A = A.row_join(zeros(m, len(g)))
+    A = A.col_join(zeros(B.rows, m).row_join(B))
+
+    return h, A
 
 
 def param_rischDE(fa, fd, G, DE):
     """
     Solve a Parametric Risch Differential Equation: Dy + f*y == Sum(ci*Gi, (i, 1, m)).
-    """
-    _, (fa, fd) = weak_normalizer(fa, fd, DE)
-    a, (ba, bd), G, hn = prde_normal_denom(ga, gd, G, DE)
-    A, B, G, hs = prde_special_denom(a, ba, bd, G, DE)
-    g = gcd(A, B)
-    A, B, G = A.quo(g), B.quo(g), [gia.cancel(gid*g, include=True) for
-        gia, gid in G]
-    Q, M = prde_linear_constraints(A, B, G, DE)
-    M, _ = constant_system(M, zeros(M.rows, 1), DE)
-    # Reduce number of constants at this point
-    try:
-        # Similar to rischDE(), we try oo, even though it might lead to
-        # non-termination when there is no solution.  At least for prde_spde,
-        # it will always terminate no matter what n is.
-        n = bound_degree(A, B, G, DE, parametric=True)
-    except NotImplementedError:
-        debug("param_rischDE: Proceeding with n = oo; may cause "
-              "non-termination.")
-        n = oo
 
-    A, B, Q, R, n1 = prde_spde(A, B, Q, n, DE)
+    Given a derivation D in k(t), f in k(t), and G
+    = [G1, ..., Gm] in k(t)^m, return h = [h1, ..., hr] in k(t)^r and
+    a matrix A with m + r columns and entries in Const(k) such that
+    Dy + f*y = Sum(ci*Gi, (i, 1, m)) has a solution y
+    in k(t) with c1, ..., cm in Const(k) if and only if y = Sum(dj*hj,
+    (j, 1, r)) where d1, ..., dr are in Const(k) and (c1, ..., cm,
+    d1, ..., dr) is a solution of Ax == 0.
+
+    Elements of k(t) are tuples (a, d) with a and d in k[t].
+    """
+    m = len(G)
+    q, (fa, fd) = weak_normalizer(fa, fd, DE)
+    # Solutions of the weakly normalized equation Dz + f*z = q*Sum(ci*Gi)
+    # correspond to solutions y = z/q of the original equation.
+    gamma = q
+    G = [(q*ga).cancel(gd, include=True) for ga, gd in G]
+
+    a, (ba, bd), G, hn = prde_normal_denom(fa, fd, G, DE)
+    # Solutions q in k<t> of  a*Dq + b*q = Sum(ci*Gi) correspond
+    # to solutions z = q/hn of the weakly normalized equation.
+    gamma *= hn
+
+    A, B, G, hs = prde_special_denom(a, ba, bd, G, DE)
+    # Solutions p in k[t] of  A*Dp + B*p = Sum(ci*Gi) correspond
+    # to solutions q = p/hs of the previous equation.
+    gamma *= hs
+
+    g = A.gcd(B)
+    a, b, g = A.quo(g), B.quo(g), [gia.cancel(gid*g, include=True) for
+        gia, gid in G]
+
+    # a*Dp + b*p = Sum(ci*gi)  may have a polynomial solution
+    # only if the sum is in k[t].
+
+    q, M = prde_linear_constraints(a, b, g, DE)
+
+    # q = [q1, ..., qm] where qi in k[t] is the polynomial component
+    # of the partial fraction expansion of gi.
+    # M is a matrix with m columns and entries in k.
+    # Sum(fi*gi, (i, 1, m)), where f1, ..., fm are elements of k,
+    # is a polynomial if and only if M*Matrix([f1, ..., fm]) == 0,
+    # in which case the sum is equal to Sum(fi*qi).
+
+    M, _ = constant_system(M, zeros(M.rows, 1), DE)
+    # M is a matrix with m columns and entries in Const(k).
+    # Sum(ci*gi) is in k[t] for c1, ..., cm in Const(k)
+    # if and only if M*Matrix([c1, ..., cm]) == 0,
+    # in which case the sum is Sum(ci*qi).
+
+    ## Reduce number of constants at this point
+
+    V = M.nullspace()
+    # V = [v1, ..., vu] where each vj is a column matrix with
+    # entries aj1, ..., ajm in Const(k).
+    # Sum(aji*gi) is in k[t] and equal to Sum(aji*qi) (j = 1, ..., u).
+    # Sum(ci*gi) is in k[t] if and only is ci = Sum(dj*aji)
+    # (i = 1, ..., m) for some d1, ..., du in Const(k).
+    # In that case,
+    #     Sum(ci*gi) = Sum(ci*qi) = Sum(dj*Sum(aji*qi)) = Sum(dj*rj)
+    # where rj = Sum(aji*qi) (j = 1, ..., u) in k[t].
+
+    if not V:  # No non-trivial solution
+        return [], eye(m)
+
+    Mq = Matrix([q])  # A single row.
+    r = [(Mq*vj)[0] for vj in V]  # [r1, ..., ru]
+
+    # Solutions of a*Dp + b*p = Sum(dj*rj) correspond to solutions
+    # y = p/gamma of the initial equation with ci = Sum(dj*aji).
+
+    try:
+        # We try n=5. At least for prde_spde, it will always
+        # terminate no matter what n is.
+        n = bound_degree(a, b, r, DE, parametric=True)
+    except NotImplementedError:
+        # A temporary bound is set. Eventually, it will be removed.
+        # the currently added test case takes large time
+        # even with n=5, and much longer with large n's.
+        n = 5
+
+    h, B = param_poly_rischDE(a, b, r, n, DE)
+
+    # h = [h1, ..., hv] in k[t]^v and and B is a matrix with u + v
+    # columns and entries in Const(k) such that
+    # a*Dp + b*p = Sum(dj*rj) has a solution p of degree <= n
+    # in k[t] if and only if p = Sum(ek*hk) where e1, ..., ev are in
+    # Const(k) and B*Matrix([d1, ..., du, e1, ..., ev]) == 0.
+    # The solutions of the original equation for ci = Sum(dj*aji)
+    # (i = 1, ..., m) are then y = Sum(ek*hk, (k, 1, v))/gamma.
+
+    ## Build combined relation matrix with m + u + v columns.
+
+    A = -eye(m)
+    for vj in V:
+        A = A.row_join(vj)
+    A = A.row_join(zeros(m, len(h)))
+    A = A.col_join(zeros(B.rows, m).row_join(B))
+
+    ## Eliminate d1, ..., du.
+
+    W = A.nullspace()
+
+    # W = [w1, ..., wt] where each wl is a column matrix with
+    # entries blk (k = 1, ..., m + u + v) in Const(k).
+    # The vectors (bl1, ..., blm) generate the space of those
+    # constant families (c1, ..., cm) for which a solution of
+    # the equation Dy + f*y == Sum(ci*Gi) exists. They generate
+    # the space and form a basis except possibly when Dy + f*y == 0
+    # is solvable in k(t}. The corresponding solutions are
+    # y = Sum(blk'*hk, (k, 1, v))/gamma, where k' = k + m + u.
+
+    v = len(h)
+    M = Matrix([wl[:m] + wl[-v:] for wl in W])  # excise dj's.
+    N = M.nullspace()
+    # N = [n1, ..., ns] where the ni in Const(k)^(m + v) are column
+    # vectors generating the space of linear relations between
+    # c1, ..., cm, e1, ..., ev.
+
+    C = Matrix([ni[:] for ni in N])  # rows n1, ..., ns.
+
+    return [hk.cancel(gamma, include=True) for hk in h], C
 
 
 def limited_integrate_reduce(fa, fd, G, DE):
@@ -422,7 +776,7 @@ def limited_integrate_reduce(fa, fd, G, DE):
     if DE.case in ['base', 'primitive', 'exp', 'tan']:
         hs = reduce(lambda i, j: i.lcm(j), (ds,) + Es)  # lcm(ds, es1, ..., esm)
         a = hn*hs
-        b = -derivation(hn, DE) - (hn*derivation(hs, DE)).quo(hs)
+        b -= (hn*derivation(hs, DE)).quo(hs)
         mu = min(order_at_oo(fa, fd, DE.t), min([order_at_oo(ga, gd, DE.t) for
             ga, gd in G]))
         # So far, all the above are also nonlinear or Liouvillian, but if this
@@ -442,35 +796,30 @@ def limited_integrate(fa, fd, G, DE):
     Solves the limited integration problem:  f = Dv + Sum(ci*wi, (i, 1, n))
     """
     fa, fd = fa*Poly(1/fd.LC(), DE.t), fd.monic()
-    A, B, h, N, g, V = limited_integrate_reduce(fa, fd, G, DE)
-    V = [g] + V
-    g = A.gcd(B)
-    A, B, V = A.quo(g), B.quo(g), [via.cancel(vid*g, include=True) for
-        via, vid in V]
-    Q, M = prde_linear_constraints(A, B, V, DE)
-    M, _ = constant_system(M, zeros(M.rows, 1), DE)
-    l = M.nullspace()
-    if M == Matrix() or len(l) > 1:
-        # Continue with param_rischDE()
-        raise NotImplementedError("param_rischDE() is required to solve this "
-            "integral.")
-    elif len(l) == 0:
-        raise NonElementaryIntegralException
-    elif len(l) == 1:
-        # The c1 == 1.  In this case, we can assume a normal Risch DE
-        if l[0][0].is_zero:
-            raise NonElementaryIntegralException
-        else:
-            l[0] *= 1/l[0][0]
-            C = sum([Poly(i, DE.t)*q for (i, q) in zip(l[0], Q)])
-            # Custom version of rischDE() that uses the already computed
-            # denominator and degree bound from above.
-            B, C, m, alpha, beta = spde(A, B, C, N, DE)
-            y = solve_poly_rde(B, C, m, DE)
-
-            return ((alpha*y + beta, h), list(l[0][1:]))
+    # interpreting limited integration problem as a
+    # parametric Risch DE problem
+    Fa = Poly(0, DE.t)
+    Fd = Poly(1, DE.t)
+    G = [(fa, fd)] + G
+    h, A = param_rischDE(Fa, Fd, G, DE)
+    V = A.nullspace()
+    V = [v for v in V if v[0] != 0]
+    if not V:
+        return None
     else:
-        raise NotImplementedError
+        # we can take any vector from V, we take V[0]
+        c0 = V[0][0]
+        # v = [-1, c1, ..., cm, d1, ..., dr]
+        v = V[0]/(-c0)
+        r = len(h)
+        m = len(v) - r - 1
+        C = list(v[1: m + 1])
+        y = -sum([v[m + 1 + i]*h[i][0].as_expr()/h[i][1].as_expr() \
+                for i in range(r)])
+        y_num, y_den = y.as_numer_denom()
+        Ya, Yd = Poly(y_num, DE.t), Poly(y_den, DE.t)
+        Y = Ya*Poly(1/Yd.LC(), DE.t), Yd.monic()
+        return Y, C
 
 
 def parametric_log_deriv_heu(fa, fd, wa, wd, DE, c1=None):
@@ -504,8 +853,7 @@ def parametric_log_deriv_heu(fa, fd, wa, wd, DE, c1=None):
             # deg(q) > B, no solution for c.
             return None
 
-        N, M = s[c1].as_numer_denom()  # N and M are integers
-        N, M = Poly(N, DE.t), Poly(M, DE.t)
+        M, N = s[c1].as_numer_denom()
 
         nfmwa = N*fa*wd - M*wa*fd
         nfmwd = fd*wd
@@ -515,9 +863,7 @@ def parametric_log_deriv_heu(fa, fd, wa, wd, DE, c1=None):
             # (N*f - M*w) is not the logarithmic derivative of a k(t)-radical.
             return None
 
-        Q, e, v = Qv
-        if e != 1:
-            return None
+        Q, v = Qv
 
         if Q.is_zero or v.is_zero:
             return None
@@ -533,8 +879,9 @@ def parametric_log_deriv_heu(fa, fd, wa, wd, DE, c1=None):
     z = ls*ln.gcd(ln.diff(DE.t))
 
     if not z.has(DE.t):
-        raise NotImplementedError("parametric_log_deriv_heu() "
-            "heuristic failed: z in k.")
+        # TODO: We treat this as 'no solution', until the structure
+        # theorem version of parametric_log_deriv is implemented.
+        return None
 
     u1, r1 = (fa*l.quo(fd)).div(z)  # (l*f).div(z)
     u2, r2 = (wa*l.quo(wd)).div(z)  # (l*w).div(z)
@@ -575,7 +922,7 @@ def parametric_log_deriv(fa, fd, wa, wd, DE):
 
 
 def is_deriv_k(fa, fd, DE):
-    """
+    r"""
     Checks if Df/f is the derivative of an element of k(t).
 
     a in k(t) is the derivative of an element of k(t) if there exists b in k(t)
@@ -625,15 +972,21 @@ def is_deriv_k(fa, fd, DE):
     of the logarithmic terms in L_args.
 
     To handle the case where we are given Df/f, not f, use is_deriv_k_in_field().
+
+    See also
+    ========
+    is_log_deriv_k_t_radical_in_field, is_log_deriv_k_t_radical
+
     """
     # Compute Df/f
-    dfa, dfd = fd*(fd*derivation(fa, DE) - fa*derivation(fd, DE)), fd**2*fa
+    dfa, dfd = (fd*derivation(fa, DE) - fa*derivation(fd, DE)), fd*fa
     dfa, dfd = dfa.cancel(dfd, include=True)
 
     # Our assumption here is that each monomial is recursively transcendental
-    if len(DE.L_K) + len(DE.E_K) != len(DE.D) - 1:
+    if len(DE.exts) != len(DE.D):
         if [i for i in DE.cases if i == 'tan'] or \
-                set([i for i in DE.cases if i == 'primitive']) - set(DE.L_K):
+                (set([i for i in DE.cases if i == 'primitive']) -
+                        set(DE.indices('log'))):
             raise NotImplementedError("Real version of the structure "
                 "theorems with hypertangent support is not yet implemented.")
 
@@ -641,8 +994,8 @@ def is_deriv_k(fa, fd, DE):
         raise NotImplementedError("Nonelementary extensions not supported "
             "in the structure theorems.")
 
-    E_part = [DE.D[i].quo(Poly(DE.T[i], DE.T[i])).as_expr() for i in DE.E_K]
-    L_part = [DE.D[i].as_expr() for i in DE.L_K]
+    E_part = [DE.D[i].quo(Poly(DE.T[i], DE.T[i])).as_expr() for i in DE.indices('exp')]
+    L_part = [DE.D[i].as_expr() for i in DE.indices('log')]
 
     lhs = Matrix([E_part + L_part])
     rhs = Matrix([dfa.as_expr()/dfd.as_expr()])
@@ -660,23 +1013,30 @@ def is_deriv_k(fa, fd, DE):
             raise NotImplementedError("Cannot work with non-rational "
                 "coefficients in this case.")
         else:
-            terms = DE.E_args + [DE.T[i] for i in DE.L_K]
+            terms = ([DE.extargs[i] for i in DE.indices('exp')] +
+                    [DE.T[i] for i in DE.indices('log')])
             ans = list(zip(terms, u))
             result = Add(*[Mul(i, j) for i, j in ans])
-            argterms = [DE.T[i] for i in DE.E_K] + DE.L_args
+            argterms = ([DE.T[i] for i in DE.indices('exp')] +
+                    [DE.extargs[i] for i in DE.indices('log')])
             l = []
+            ld = []
             for i, j in zip(argterms, u):
                 # We need to get around things like sqrt(x**2) != x
                 # and also sqrt(x**2 + 2*x + 1) != x + 1
+                # Issue 10798: i need not be a polynomial
+                i, d = i.as_numer_denom()
                 icoeff, iterms = sqf_list(i)
                 l.append(Mul(*([Pow(icoeff, j)] + [Pow(b, e*j) for b, e in iterms])))
-            const = cancel(fa.as_expr()/fd.as_expr()/Mul(*l))
+                dcoeff, dterms = sqf_list(d)
+                ld.append(Mul(*([Pow(dcoeff, j)] + [Pow(b, e*j) for b, e in dterms])))
+            const = cancel(fa.as_expr()/fd.as_expr()/Mul(*l)*Mul(*ld))
 
             return (ans, result, const)
 
 
 def is_log_deriv_k_t_radical(fa, fd, DE, Df=True):
-    """
+    r"""
     Checks if Df is the logarithmic derivative of a k(t)-radical.
 
     b in k(t) can be written as the logarithmic derivative of a k(t) radical if
@@ -726,8 +1086,12 @@ def is_log_deriv_k_t_radical(fa, fd, DE, Df=True):
 
     To handle the case where we are given Df, not f, use
     is_log_deriv_k_t_radical_in_field().
+
+    See also
+    ========
+    is_log_deriv_k_t_radical_in_field, is_deriv_k
+
     """
-    H = []
     if Df:
         dfa, dfd = (fd*derivation(fa, DE) - fa*derivation(fd, DE)).cancel(fd**2,
             include=True)
@@ -735,9 +1099,10 @@ def is_log_deriv_k_t_radical(fa, fd, DE, Df=True):
         dfa, dfd = fa, fd
 
     # Our assumption here is that each monomial is recursively transcendental
-    if len(DE.L_K) + len(DE.E_K) != len(DE.D) - 1:
+    if len(DE.exts) != len(DE.D):
         if [i for i in DE.cases if i == 'tan'] or \
-                set([i for i in DE.cases if i == 'primitive']) - set(DE.L_K):
+                (set([i for i in DE.cases if i == 'primitive']) -
+                        set(DE.indices('log'))):
             raise NotImplementedError("Real version of the structure "
                 "theorems with hypertangent support is not yet implemented.")
 
@@ -745,8 +1110,8 @@ def is_log_deriv_k_t_radical(fa, fd, DE, Df=True):
         raise NotImplementedError("Nonelementary extensions not supported "
             "in the structure theorems.")
 
-    E_part = [DE.D[i].quo(Poly(DE.T[i], DE.T[i])).as_expr() for i in DE.E_K]
-    L_part = [DE.D[i].as_expr() for i in DE.L_K]
+    E_part = [DE.D[i].quo(Poly(DE.T[i], DE.T[i])).as_expr() for i in DE.indices('exp')]
+    L_part = [DE.D[i].as_expr() for i in DE.indices('log')]
 
     lhs = Matrix([E_part + L_part])
     rhs = Matrix([dfa.as_expr()/dfd.as_expr()])
@@ -768,13 +1133,15 @@ def is_log_deriv_k_t_radical(fa, fd, DE, Df=True):
         else:
             n = reduce(ilcm, [i.as_numer_denom()[1] for i in u])
             u *= n
-            terms = [DE.T[i] for i in DE.E_K] + DE.L_args
+            terms = ([DE.T[i] for i in DE.indices('exp')] +
+                    [DE.extargs[i] for i in DE.indices('log')])
             ans = list(zip(terms, u))
             result = Mul(*[Pow(i, j) for i, j in ans])
 
             # exp(f) will be the same as result up to a multiplicative
             # constant.  We now find the log of that constant.
-            argterms = DE.E_args + [DE.T[i] for i in DE.L_K]
+            argterms = ([DE.extargs[i] for i in DE.indices('exp')] +
+                    [DE.T[i] for i in DE.indices('log')])
             const = cancel(fa.as_expr()/fd.as_expr() -
                 Add(*[Mul(i, j/n) for i, j in zip(argterms, u)]))
 
@@ -785,6 +1152,11 @@ def is_log_deriv_k_t_radical_in_field(fa, fd, DE, case='auto', z=None):
     """
     Checks if f can be written as the logarithmic derivative of a k(t)-radical.
 
+    It differs from is_log_deriv_k_t_radical(fa, fd, DE, Df=False)
+    for any given fa, fd, DE in that it finds the solution in the
+    given field not in some (possibly unspecified extension) and
+    "in_field" with the function name is used to indicate that.
+
     f in k(t) can be written as the logarithmic derivative of a k(t) radical if
     there exist n in ZZ and u in k(t) with n, u != 0 such that n*f == Du/u.
     Either returns (n, u) or None, which means that f cannot be written as the
@@ -793,6 +1165,11 @@ def is_log_deriv_k_t_radical_in_field(fa, fd, DE, case='auto', z=None):
     case is one of {'primitive', 'exp', 'tan', 'auto'} for the primitive,
     hyperexponential, and hypertangent cases, respectively.  If case is 'auto',
     it will attempt to determine the type of the derivation automatically.
+
+    See also
+    ========
+    is_log_deriv_k_t_radical, is_deriv_k
+
     """
     fa, fd = fa.cancel(fd, include=True)
 
@@ -869,7 +1246,7 @@ def is_log_deriv_k_t_radical_in_field(fa, fd, DE, case='auto', z=None):
             return None
         # Note: if residueterms = [], returns (1, 1)
         # f had better be 0 in that case.
-        n = reduce(ilcm, [i.as_numer_denom()[1] for _, i in residueterms], S(1))
+        n = reduce(ilcm, [i.as_numer_denom()[1] for _, i in residueterms], S.One)
         u = Mul(*[Pow(i, j*n) for i, j in residueterms])
         return (n, u)
 
@@ -886,7 +1263,7 @@ def is_log_deriv_k_t_radical_in_field(fa, fd, DE, case='auto', z=None):
         "'base', 'auto'}, not %s" % case)
 
     common_denom = reduce(ilcm, [i.as_numer_denom()[1] for i in [j for _, j in
-        residueterms]] + [n], S(1))
+        residueterms]] + [n], S.One)
     residueterms = [(i, j*common_denom) for i, j in residueterms]
     m = common_denom//n
     if common_denom != n*m:  # Verify exact division
