@@ -35,28 +35,53 @@ def _init_python_printing(stringify_func, **settings):
 
 def _init_ipython_printing(ip, stringify_func, use_latex, euler, forecolor,
                            backcolor, fontsize, latex_mode, print_builtin,
-                           latex_printer, **settings):
+                           latex_printer, scale, **settings):
     """Setup printing in IPython interactive session. """
     try:
         from IPython.lib.latextools import latex_to_png
     except ImportError:
         pass
 
+    # Guess best font color if none was given based on the ip.colors string.
+    # From the IPython documentation:
+    #   It has four case-insensitive values: 'nocolor', 'neutral', 'linux',
+    #   'lightbg'. The default is neutral, which should be legible on either
+    #   dark or light terminal backgrounds. linux is optimised for dark
+    #   backgrounds and lightbg for light ones.
+    if forecolor is None:
+        color = ip.colors.lower()
+        if color == 'lightbg':
+            forecolor = 'Black'
+        elif color == 'linux':
+            forecolor = 'White'
+        else:
+            # No idea, go with gray.
+            forecolor = 'Gray'
+        debug("init_printing: Automatic foreground color:", forecolor)
+
     preamble = "\\documentclass[varwidth,%s]{standalone}\n" \
                "\\usepackage{amsmath,amsfonts}%s\\begin{document}"
     if euler:
         addpackages = '\\usepackage{euler}'
     else:
         addpackages = ''
+    if use_latex == "svg":
+        addpackages = addpackages + "\n\\special{color %s}" % forecolor
+
     preamble = preamble % (fontsize, addpackages)
 
     imagesize = 'tight'
     offset = "0cm,0cm"
-    resolution = 150
+    resolution = round(150*scale)
     dvi = r"-T %s -D %d -bg %s -fg %s -O %s" % (
         imagesize, resolution, backcolor, forecolor, offset)
     dvioptions = dvi.split()
+
+    svg_scale = 150/72*scale
+    dvioptions_svg = ["--no-fonts", "--scale={}".format(svg_scale)]
+
     debug("init_printing: DVIOPTIONS:", dvioptions)
+    debug("init_printing: DVIOPTIONS_SVG:", dvioptions_svg)
     debug("init_printing: PREAMBLE:", preamble)
 
     latex = latex_printer or default_latex
@@ -81,6 +106,19 @@ def _init_ipython_printing(ip, stringify_func, use_latex, euler, forecolor,
             raise
         return exprbuffer.getvalue()
 
+    def _svg_wrapper(o):
+        exprbuffer = BytesIO()
+        try:
+            preview(o, output='svg', viewer='BytesIO',
+                    outputbuffer=exprbuffer, preamble=preamble,
+                    dvioptions=dvioptions_svg)
+        except Exception as e:
+            # IPython swallows exceptions
+            debug("svg printing:", "_preview_wrapper exception raised:",
+                  repr(e))
+            raise
+        return exprbuffer.getvalue().decode('utf-8')
+
     def _matplotlib_wrapper(o):
         # mathtext does not understand certain latex flags, so we try to
         # replace them with suitable subs
@@ -90,7 +128,10 @@ def _init_ipython_printing(ip, stringify_func, use_latex, euler, forecolor,
         # render any LaTeX environments such as array or matrix. So here we
         # ensure that if mathtext fails to render, we return None.
         try:
-            return latex_to_png(o)
+            try:
+                return latex_to_png(o, color=forecolor, scale=scale)
+            except TypeError: #  Old IPython version without color and scale
+                return latex_to_png(o)
         except ValueError as e:
             debug('matplotlib exception caught:', repr(e))
             return None
@@ -159,6 +200,21 @@ def _init_ipython_printing(ip, stringify_func, use_latex, euler, forecolor,
                     s = latex(o, mode='inline', **settings)
                 return _matplotlib_wrapper(s)
 
+    def _print_latex_svg(o):
+        """
+        A function that returns a svg rendered by an external latex
+        distribution, no fallback available.
+        """
+        if _can_print_latex(o):
+            s = latex(o, mode=latex_mode, **settings)
+            if latex_mode == 'plain':
+                s = '$\\displaystyle %s$' % s
+            try:
+                return _svg_wrapper(s)
+            except RuntimeError as e:
+                debug('preview failed with:', repr(e),
+                      ' No fallback available.')
+
     def _print_latex_matplotlib(o):
         """
         A function that returns a png rendered by mathtext
@@ -210,6 +266,19 @@ def _init_ipython_printing(ip, stringify_func, use_latex, euler, forecolor,
         for cls in printable_types:
             plaintext_formatter.for_type(cls, _print_plain)
 
+        svg_formatter = ip.display_formatter.formatters['image/svg+xml']
+        if use_latex in ('svg', ):
+            debug("init_printing: using svg formatter")
+            for cls in printable_types:
+                svg_formatter.for_type(cls, _print_latex_svg)
+        else:
+            debug("init_printing: not using any svg formatter")
+            for cls in printable_types:
+                # Better way to set this, but currently does not work in IPython
+                #png_formatter.for_type(cls, None)
+                if cls in svg_formatter.type_printers:
+                    svg_formatter.type_printers.pop(cls)
+
         png_formatter = ip.display_formatter.formatters['image/png']
         if use_latex in (True, 'png'):
             debug("init_printing: using png formatter")
@@ -270,77 +339,90 @@ NO_GLOBAL = False
 
 def init_printing(pretty_print=True, order=None, use_unicode=None,
                   use_latex=None, wrap_line=None, num_columns=None,
-                  no_global=False, ip=None, euler=False, forecolor='Black',
+                  no_global=False, ip=None, euler=False, forecolor=None,
                   backcolor='Transparent', fontsize='10pt',
                   latex_mode='plain', print_builtin=True,
                   str_printer=None, pretty_printer=None,
-                  latex_printer=None, **settings):
+                  latex_printer=None, scale=1.0, **settings):
     r"""
     Initializes pretty-printer depending on the environment.
 
     Parameters
     ==========
 
-    pretty_print: boolean
+    pretty_print : boolean, default=True
         If True, use pretty_print to stringify or the provided pretty
         printer; if False, use sstrrepr to stringify or the provided string
         printer.
-    order: string or None
+    order : string or None, default='lex'
         There are a few different settings for this parameter:
         lex (default), which is lexographic order;
         grlex, which is graded lexographic order;
         grevlex, which is reversed graded lexographic order;
         old, which is used for compatibility reasons and for long expressions;
         None, which sets it to lex.
-    use_unicode: boolean or None
+    use_unicode : boolean or None, default=None
         If True, use unicode characters;
-        if False, do not use unicode characters.
-    use_latex: string, boolean, or None
-        If True, use default latex rendering in GUI interfaces (png and
+        if False, do not use unicode characters;
+        if None, make a guess based on the environment.
+    use_latex : string, boolean, or None, default=None
+        If True, use default LaTeX rendering in GUI interfaces (png and
         mathjax);
-        if False, do not use latex rendering;
+        if False, do not use LaTeX rendering;
+        if None, make a guess based on the environment;
         if 'png', enable latex rendering with an external latex compiler,
         falling back to matplotlib if external compilation fails;
-        if 'matplotlib', enable latex rendering with matplotlib;
-        if 'mathjax', enable latex text generation, for example MathJax
-        rendering in IPython notebook or text rendering in LaTeX documents
-    wrap_line: boolean
+        if 'matplotlib', enable LaTeX rendering with matplotlib;
+        if 'mathjax', enable LaTeX text generation, for example MathJax
+        rendering in IPython notebook or text rendering in LaTeX documents;
+        if 'svg', enable LaTeX rendering with an external latex compiler,
+        no fallback
+    wrap_line : boolean
         If True, lines will wrap at the end; if False, they will not wrap
-        but continue as one line. This is only relevant if `pretty_print` is
+        but continue as one line. This is only relevant if ``pretty_print`` is
         True.
-    num_columns: int or None
+    num_columns : int or None, default=None
         If int, number of columns before wrapping is set to num_columns; if
         None, number of columns before wrapping is set to terminal width.
-        This is only relevant if `pretty_print` is True.
-    no_global: boolean
+        This is only relevant if ``pretty_print`` is True.
+    no_global : boolean, default=False
         If True, the settings become system wide;
         if False, use just for this console/session.
-    ip: An interactive console
+    ip : An interactive console
         This can either be an instance of IPython,
         or a class that derives from code.InteractiveConsole.
-    euler: boolean, optional, default=False
+    euler : boolean, optional, default=False
         Loads the euler package in the LaTeX preamble for handwritten style
         fonts (http://www.ctan.org/pkg/euler).
-    forecolor: string, optional, default='Black'
-        DVI setting for foreground color.
-    backcolor: string, optional, default='Transparent'
-        DVI setting for background color.
-    fontsize: string, optional, default='10pt'
+    forecolor : string or None, optional, default=None
+        DVI setting for foreground color. None means that either 'Black',
+        'White', or 'Gray' will be selected based on a guess of the IPython
+        terminal color setting. See notes.
+    backcolor : string, optional, default='Transparent'
+        DVI setting for background color. See notes.
+    fontsize : string, optional, default='10pt'
         A font size to pass to the LaTeX documentclass function in the
-        preamble.
-    latex_mode: string, optional, default='plain'
+        preamble. Note that the options are limited by the documentclass.
+        Consider using scale instead.
+    latex_mode : string, optional, default='plain'
         The mode used in the LaTeX printer. Can be one of:
         {'inline'|'plain'|'equation'|'equation*'}.
-    print_builtin: boolean, optional, default=True
-        If true then floats and integers will be printed. If false the
+    print_builtin : boolean, optional, default=True
+        If ``True`` then floats and integers will be printed. If ``False`` the
         printer will only print SymPy types.
-    str_printer: function, optional, default=None
+    str_printer : function, optional, default=None
         A custom string printer function. This should mimic
         sympy.printing.sstrrepr().
-    pretty_printer: function, optional, default=None
+    pretty_printer : function, optional, default=None
         A custom pretty printer. This should mimic sympy.printing.pretty().
-    latex_printer: function, optional, default=None
+    latex_printer : function, optional, default=None
         A custom LaTeX printer. This should mimic sympy.printing.latex().
+    scale : float, optional, default=1.0
+        Scale the LaTeX output when using the ``png`` or ``svg`` backends.
+        Useful for high dpi screens.
+    settings :
+        Any additional settings for the ``latex`` and ``pretty`` commands can
+        be used to fine-tune the output.
 
     Examples
     ========
@@ -377,6 +459,38 @@ def init_printing(pretty_print=True, order=None, use_unicode=None,
     >>> x**2 + x + y**2 + y # doctest: +SKIP
     x + y +
     x**2 + y**2
+
+    Notes
+    =====
+
+    The foreground and background colors can be selected when using 'png' or
+    'svg' LaTeX rendering. Note that before the ``init_printing`` command is
+    executed, the LaTeX rendering is handled by the IPython console and not SymPy.
+
+    The colors can be selected among the 68 standard colors known to ``dvips``,
+    for a list see [1]_. In addition, the background color can be
+    set to  'Transparent' (which is the default value).
+
+    When using the 'Auto' foreground color, the guess is based on the
+    ``colors`` variable in the IPython console, see [2]_. Hence, if
+    that variable is set correctly in your IPython console, there is a high
+    chance that the output will be readable, although manual settings may be
+    needed.
+
+
+    References
+    ==========
+
+    .. [1] https://en.wikibooks.org/wiki/LaTeX/Colors#The_68_standard_colors_known_to_dvips
+
+    .. [2] https://ipython.readthedocs.io/en/stable/config/details.html#terminal-colors
+
+    See Also
+    ========
+
+    sympy.printing.latex
+    sympy.printing.pretty
+
     """
     import sys
     from sympy.printing.printer import Printer
@@ -451,6 +565,7 @@ def init_printing(pretty_print=True, order=None, use_unicode=None,
                   "of IPython printing")
         _init_ipython_printing(ip, stringify_func, use_latex, euler,
                                forecolor, backcolor, fontsize, latex_mode,
-                               print_builtin, latex_printer, **settings)
+                               print_builtin, latex_printer, scale,
+                               **settings)
     else:
         _init_python_printing(stringify_func, **settings)
